@@ -150,26 +150,6 @@ status_message (GtkWidget       *widget,
     gtk_statusbar_push (GTK_STATUSBAR (guw->status), 0, message);
 }
 
-static gboolean
-update_progress_bar (GucharmapWindow *guw)
-{
-  gdouble fraction_completed;
-
-  fraction_completed = gucharmap_search_dialog_get_completed (GUCHARMAP_SEARCH_DIALOG (guw->search_dialog));
-
-  if (fraction_completed < 0 || fraction_completed > 1)
-    {
-      gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (guw->progress), 0);
-      gtk_progress_bar_set_text (GTK_PROGRESS_BAR (guw->progress), NULL);
-      return FALSE;
-    }
-  else
-    {
-      gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (guw->progress), fraction_completed);
-      return TRUE;
-    }
-}
-
 static void
 search_start (GucharmapSearchDialog *search_dialog,
               GucharmapWindow       *guw)
@@ -179,7 +159,7 @@ search_start (GucharmapSearchDialog *search_dialog,
 
   cursor = gdk_cursor_new_for_display (gtk_widget_get_display (GTK_WIDGET (guw)), GDK_WATCH);
   gdk_window_set_cursor (gtk_widget_get_window (GTK_WIDGET (guw)), cursor);
-  gdk_cursor_unref (cursor);
+  g_object_unref (cursor);
 
   action = gtk_action_group_get_action (guw->action_group, "Find");
   gtk_action_set_sensitive (action, FALSE);
@@ -187,9 +167,6 @@ search_start (GucharmapSearchDialog *search_dialog,
   gtk_action_set_sensitive (action, FALSE);
   action = gtk_action_group_get_action (guw->action_group, "FindPrevious");
   gtk_action_set_sensitive (action, FALSE);
-
-  gtk_progress_bar_set_text (GTK_PROGRESS_BAR (guw->progress), _("Searching…"));
-  g_timeout_add (100, (GSourceFunc) update_progress_bar, guw);
 }
 
 static void
@@ -199,9 +176,6 @@ search_finish (GucharmapSearchDialog *search_dialog,
 {
   GtkAction *action;
 
-  gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (guw->progress), 0);
-  gtk_progress_bar_set_text (GTK_PROGRESS_BAR (guw->progress), NULL);
-
   if (found_char != (gunichar)(-1))
     gucharmap_charmap_set_active_character (guw->charmap, found_char);
   /* not-found dialog handled by GucharmapSearchDialog */
@@ -331,7 +305,7 @@ snap_cols_pow2 (GtkAction        *action,
 {
   gboolean is_active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
   gucharmap_charmap_set_snap_pow2 (guw->charmap, is_active);
-  gucharmap_settings_set_snap_pow2 (is_active);
+  g_settings_set_boolean (guw->settings, "snap-cols-pow2", is_active);
 }
 
 static void
@@ -449,8 +423,7 @@ help_about (GtkAction       *action,
   gtk_show_about_dialog (GTK_WINDOW (guw),
 			 "program-name", _("GNOME Character Map"),
 			 "version", VERSION,
-			 "comments", _("Based on the Unicode Character Database 6.1.0 (Beta)"),
-                         /* _("Based on the Unicode Character Database 6.1.0"), */
+			 "comments", _("Based on the Unicode Character Database 6.1.0"),
 			 "copyright", "Copyright © 2004 Noah Levitt\n"
 				      "Copyright © 1991–2010 Unicode, Inc.\n"
 				      "Copyright © 2007–2010 Christian Persch",
@@ -472,7 +445,6 @@ next_or_prev_character (GtkAction       *action,
 {
   GucharmapChartable *chartable;
   GucharmapChartableClass *klass;
-  GtkBindingSet *binding_set;
   const char *name;
   guint keyval = 0;
 
@@ -485,7 +457,6 @@ next_or_prev_character (GtkAction       *action,
 
   chartable = gucharmap_charmap_get_chartable (guw->charmap);
   klass = GUCHARMAP_CHARTABLE_GET_CLASS (chartable);
-  binding_set = gtk_binding_set_by_class (klass);
   gtk_binding_set_activate (gtk_binding_set_by_class (klass),
                             keyval,
                             0,
@@ -572,7 +543,7 @@ view_by (GtkAction        *action,
     }
 
   gucharmap_window_set_chapters_model (guw, mode);
-  gucharmap_settings_set_chapters_mode (mode);
+  g_settings_set_enum (guw->settings, "group-by", mode);
 }
 
 #ifdef DEBUG_chpe
@@ -702,12 +673,17 @@ status_realize (GtkWidget       *status,
 static gboolean
 save_last_char_idle_cb (GucharmapWindow *guw)
 {
+  gchar outbuf[10];
   gunichar wc;
+  gint len;
 
   guw->save_last_char_idle_id = 0;
 
   wc = gucharmap_charmap_get_active_character (guw->charmap);
-  gucharmap_settings_set_last_char (wc);
+  len = g_unichar_to_utf8 (wc, outbuf);
+  outbuf[len] = '\0';
+
+  g_settings_set_string (guw->settings, "last-char", outbuf);
 
   return FALSE;
 }
@@ -730,7 +706,7 @@ fontsel_sync_font_desc (GucharmapMiniFontSelection *fontsel,
   guw->in_notification = FALSE;
 
   font = pango_font_description_to_string (font_desc);
-  gucharmap_settings_set_font (font);
+  g_settings_set (guw->settings, "font", "ms", font);
   g_free (font);
 }
 
@@ -763,10 +739,25 @@ charmap_sync_active_character (GtkWidget *widget,
   guw->save_last_char_idle_id = g_idle_add ((GSourceFunc) save_last_char_idle_cb, guw);
 }
 
+static gboolean
+string_to_unichar_mapping (GVariant *value,
+                           gpointer *result,
+                           gpointer  user_data)
+{
+  const gchar *str;
+
+  str = g_variant_get_string (value, NULL);
+  if (g_utf8_strlen (str, -1) != 1)
+    return FALSE;
+
+  *result = GINT_TO_POINTER (g_utf8_get_char (str));
+  return TRUE;
+}
+
 static void
 gucharmap_window_init (GucharmapWindow *guw)
 {
-  GtkWidget *big_vbox, *hbox, *bbox, *button, *label;
+  GtkWidget *grid, *button, *label;
   GucharmapChartable *chartable;
   /* tooltips are NULL because they are never actually shown in the program */
   const GtkActionEntry menu_entries[] =
@@ -836,11 +827,14 @@ gucharmap_window_init (GucharmapWindow *guw)
       NULL,
       G_CALLBACK (no_font_fallback_toggled_cb), FALSE },
     { "SnapColumns", NULL, N_("Snap _Columns to Power of Two"), NULL,
-      NULL,
-      G_CALLBACK (snap_cols_pow2), FALSE },
+      NULL, NULL, FALSE },
   };
   GtkWidget *menubar;
   GtkAction *action;
+  gunichar active;
+  gchar *font;
+
+  guw->settings = g_settings_new ("org.gnome.Charmap");
 
   gtk_window_set_title (GTK_WINDOW (guw), _("Character Map"));
   gtk_window_set_icon_name (GTK_WINDOW (guw), GUCHARMAP_ICON_NAME);
@@ -861,7 +855,7 @@ gucharmap_window_init (GucharmapWindow *guw)
   gtk_action_group_add_radio_actions (guw->action_group,
   				      radio_menu_entries,
 				      G_N_ELEMENTS (radio_menu_entries),
-				      gucharmap_settings_get_chapters_mode(),
+				      g_settings_get_enum (guw->settings, "group-by"),
 				      G_CALLBACK (view_by),
 				      guw);
   gtk_action_group_add_toggle_actions (guw->action_group,
@@ -875,102 +869,101 @@ gucharmap_window_init (GucharmapWindow *guw)
   gtk_ui_manager_add_ui_from_string (guw->uimanager, ui_info, strlen (ui_info), NULL);
   
   /* Now the widgets */
-  big_vbox = gtk_vbox_new (FALSE, 0);
-  gtk_container_add (GTK_CONTAINER (guw), big_vbox);
+  grid = gtk_grid_new ();
+  gtk_container_add (GTK_CONTAINER (guw), grid);
 
   /* First the menubar */
   menubar = gtk_ui_manager_get_widget (guw->uimanager, "/MenuBar");
-  gtk_box_pack_start (GTK_BOX (big_vbox), menubar, FALSE, FALSE, 0);
+  gtk_grid_attach (GTK_GRID (grid), menubar, 0, 0, 3, 1);
 
   /* The font selector */
   guw->fontsel = gucharmap_mini_font_selection_new ();
-  g_signal_connect (guw->fontsel, "notify::font-desc",
-                    G_CALLBACK (fontsel_sync_font_desc), guw);
-  gtk_box_pack_start (GTK_BOX (big_vbox), guw->fontsel, FALSE, FALSE, 0);
+  gtk_grid_attach (GTK_GRID (grid), guw->fontsel, 0, 1, 3, 1);
   gtk_widget_show (GTK_WIDGET (guw->fontsel));
 
   /* The charmap */
   guw->charmap = GUCHARMAP_CHARMAP (gucharmap_charmap_new ());
-  g_signal_connect (guw->charmap, "notify::active-character",
-                    G_CALLBACK (charmap_sync_active_character), guw);
   g_signal_connect (guw->charmap, "notify::font-desc",
                     G_CALLBACK (charmap_sync_font_desc), guw);
 
-  gtk_box_pack_start (GTK_BOX (big_vbox), GTK_WIDGET (guw->charmap),
-                      TRUE, TRUE, 0);
+  gtk_grid_attach (GTK_GRID (grid), GTK_WIDGET (guw->charmap), 0, 2, 3, 1);
   gtk_widget_show (GTK_WIDGET (guw->charmap));
 
   /* Text to copy entry + button */
-  hbox = gtk_hbox_new (FALSE, 6);
-  gtk_container_set_border_width (GTK_CONTAINER (hbox), 6);
-
   label = gtk_label_new_with_mnemonic (_("_Text to copy:"));
-  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+  g_object_set (label, "margin", 6, NULL);
+  gtk_grid_attach (GTK_GRID (grid), label, 0, 3, 1, 1);
   gtk_widget_show (label);
 
-  bbox = gtk_hbutton_box_new ();
-
   button = gtk_button_new_from_stock (GTK_STOCK_COPY);
+  g_object_set (button, "margin", 6, NULL);
   gtk_widget_set_tooltip_text (button, _("Copy to the clipboard."));
   g_signal_connect (G_OBJECT (button), "clicked",
                     G_CALLBACK (edit_copy), guw);
-  gtk_box_pack_start (GTK_BOX (bbox), button, FALSE, FALSE, 0);
+  gtk_grid_attach (GTK_GRID (grid), button, 2, 3, 1, 1);
   gtk_widget_show (button);
 
   gtk_widget_set_sensitive (button, FALSE);
   guw->text_to_copy_entry = gtk_entry_new ();
+  g_object_set (guw->text_to_copy_entry, "margin", 6, NULL);
+  gtk_widget_set_hexpand (guw->text_to_copy_entry, TRUE);
   gtk_label_set_mnemonic_widget (GTK_LABEL (label), guw->text_to_copy_entry);
   g_signal_connect (G_OBJECT (guw->text_to_copy_entry), "changed",
                     G_CALLBACK (entry_changed_sensitize_button), button);
 
-  gtk_box_pack_start (GTK_BOX (hbox), guw->text_to_copy_entry, TRUE, TRUE, 0);
+  gtk_grid_attach (GTK_GRID (grid), guw->text_to_copy_entry, 1, 3, 1, 1);
   gtk_widget_show (guw->text_to_copy_entry);
 
-  gtk_box_pack_end (GTK_BOX (hbox), bbox, FALSE, FALSE, 0);
-  gtk_widget_show (bbox);
-
-  gtk_box_pack_start (GTK_BOX (big_vbox), hbox, FALSE, FALSE, 0);
-  gtk_widget_show (hbox);
-
   /* FIXMEchpe!! */
   chartable = gucharmap_charmap_get_chartable (guw->charmap);
   g_signal_connect (chartable, "activate", G_CALLBACK (insert_character_in_text_to_copy), guw);
 
   /* Finally the statusbar */
-  hbox = gtk_hbox_new (FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (big_vbox), hbox, FALSE, FALSE, 0);
-
   guw->status = gtk_statusbar_new ();
-  gtk_box_pack_start (GTK_BOX (hbox), guw->status, TRUE, TRUE, 0);
+  gtk_grid_attach (GTK_GRID (grid), guw->status, 0, 4, 3, 1);
   gtk_widget_show (guw->status);
   g_signal_connect (guw->status, "realize", G_CALLBACK (status_realize), guw);
 
-  guw->progress = gtk_progress_bar_new ();
-  gtk_box_pack_start (GTK_BOX (hbox), guw->progress, FALSE, FALSE, 0);
-
-#if 0
-  grip = gtk_statusbar_new ();
-  gtk_statusbar_set_has_resize_grip (GTK_STATUSBAR (grip), TRUE);
-  gtk_box_pack_start (GTK_BOX (hbox), grip, FALSE, FALSE, 0);
-#endif
-  gtk_widget_show_all (hbox);
-
   g_signal_connect (guw->charmap, "status-message", G_CALLBACK (status_message), guw);
 
-  gtk_widget_show (big_vbox);
-
-  action = gtk_action_group_get_action (guw->action_group, "SnapColumns");
-  gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action),
-                                gucharmap_settings_get_snap_pow2 ());
-
-  gucharmap_window_set_chapters_model (guw, gucharmap_settings_get_chapters_mode ());
-
-  gucharmap_charmap_set_active_character (guw->charmap,
-                                          gucharmap_settings_get_last_char ());
+  gtk_widget_show (grid);
 
   gtk_widget_grab_focus (GTK_WIDGET (gucharmap_charmap_get_chartable (guw->charmap)));
 
   gtk_window_set_has_resize_grip (GTK_WINDOW (guw), TRUE);
+
+  /* read initial settings */
+  /* font */
+  g_settings_get (guw->settings, "font", "ms", &font);
+  if (font != NULL)
+    {
+      gucharmap_window_set_font (guw, font);
+      g_free (font);
+    }
+
+  /* snap-to-power-of-two */
+  action = gtk_action_group_get_action (guw->action_group, "SnapColumns");
+  gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), g_settings_get_boolean (guw->settings, "snap-cols-pow2"));
+
+  /* group by */
+  gucharmap_window_set_chapters_model (guw, g_settings_get_enum (guw->settings, "group-by"));
+
+  /* active character */
+  active = (gsize) g_settings_get_mapped (guw->settings, "last-char", string_to_unichar_mapping, NULL);
+  gucharmap_charmap_set_active_character (guw->charmap, active);
+
+  /* window geometry */
+  gucharmap_settings_add_window (GTK_WINDOW (guw));
+
+  /* connect these only after applying the initial settings in order to
+   * avoid unnecessary writes to GSettings.
+   */
+  g_signal_connect (guw->charmap, "notify::active-character",
+                    G_CALLBACK (charmap_sync_active_character), guw);
+  g_signal_connect (guw->fontsel, "notify::font-desc",
+                    G_CALLBACK (fontsel_sync_font_desc), guw);
+  g_signal_connect (action, "activate",
+                    G_CALLBACK (snap_cols_pow2), guw);
 }
 
 static void
